#-*- coding: utf-8 -*-

from django.test import TestCase, RequestFactory
from django.core.urlresolvers import reverse
from django.core.cache import cache
from django.http import Http404
from django.core.exceptions import PermissionDenied
from django.contrib.auth.models import User as UserModel
from django.contrib.auth import get_user_model

import utils

from spirit.views.admin import user, category, comment_flag, config, index, topic
from spirit.models.category import Category
from spirit.models.comment_flag import CommentFlag, Flag
from spirit.forms.admin import UserEditForm, CategoryForm, BasicConfigForm, CommentFlagForm


User = get_user_model()


class AdminViewTest(TestCase):

    fixtures = ['spirit_init.json', ]

    def setUp(self):
        cache.clear()
        self.user = utils.create_user(is_administrator=True)
        self.category = utils.create_category()
        self.topic = utils.create_topic(self.category, user=self.user)

    def test_permission_denied_to_non_admin(self):
        req = RequestFactory().get('/')
        req.user = UserModel()
        req.user.is_administrator = False

        self.assertRaises(PermissionDenied, category.category_list, req)
        self.assertRaises(PermissionDenied, category.category_create, req)
        self.assertRaises(PermissionDenied, category.category_update, req)

        self.assertRaises(PermissionDenied, comment_flag.flag_closed, req)
        self.assertRaises(PermissionDenied, comment_flag.flag_open, req)
        self.assertRaises(PermissionDenied, comment_flag.flag_detail, req)

        self.assertRaises(PermissionDenied, config.config_basic, req)

        self.assertRaises(PermissionDenied, index.dashboard, req)

        self.assertRaises(PermissionDenied, topic.topic_deleted, req)
        self.assertRaises(PermissionDenied, topic.topic_closed, req)
        self.assertRaises(PermissionDenied, topic.topic_pinned, req)

        self.assertRaises(PermissionDenied, user.user_edit, req)
        self.assertRaises(PermissionDenied, user.user_list, req)
        self.assertRaises(PermissionDenied, user.user_admins, req)
        self.assertRaises(PermissionDenied, user.user_mods, req)
        self.assertRaises(PermissionDenied, user.user_unactive, req)

    def test_user_edit(self):
        """
        Edit user profile
        """
        utils.login(self)
        form_data = {"username": "fooedit", "email": "foo@bar.com", "location": "Bs As",
                     "timezone": "UTC", "is_administrator": True, "is_moderator": True, "is_active": True}
        response = self.client.post(reverse('spirit:admin-user-edit', kwargs={'user_id': self.user.pk, }),
                                    form_data)
        expected_url = reverse('spirit:admin-user-edit', kwargs={'user_id': self.user.pk, })
        self.assertRedirects(response, expected_url, status_code=302)

        response = self.client.get(reverse('spirit:admin-user-edit', kwargs={'user_id': self.user.pk, }))
        self.assertEqual(response.status_code, 200)

    def test_user_list(self):
        """
        List of all users
        """
        utils.login(self)
        response = self.client.get(reverse('spirit:admin-user-list'))
        self.assertQuerysetEqual(response.context['users'], map(repr, [self.user, ]))

    def test_user_admins(self):
        """
        List of admins
        """
        utils.login(self)
        response = self.client.get(reverse('spirit:admin-user-admins'))
        self.assertQuerysetEqual(response.context['users'], map(repr, [self.user, ]))

    def test_user_mods(self):
        """
        List of admins
        """
        mod = utils.create_user(is_moderator=True)
        utils.login(self)
        response = self.client.get(reverse('spirit:admin-user-mods'))
        self.assertQuerysetEqual(response.context['users'], map(repr, [mod, ]))

    def test_user_unactive(self):
        """
        List of unactive
        """
        unactive = utils.create_user()
        User.objects.filter(pk=unactive.pk).update(is_active=False)
        utils.login(self)
        response = self.client.get(reverse('spirit:admin-user-unactive'))
        self.assertQuerysetEqual(response.context['users'], map(repr, [unactive, ]))

    def test_index_dashboard(self):
        utils.login(self)
        response = self.client.get(reverse('spirit:admin-topic'))
        self.assertEqual(response.status_code, 200)

    def test_topic_deleted(self):
        """
        Deleted topics
        """
        topic_ = utils.create_topic(self.category, is_removed=True)
        utils.login(self)
        response = self.client.get(reverse('spirit:admin-topic-deleted'))
        self.assertQuerysetEqual(response.context['topics'], map(repr, [topic_, ]))

    def test_topic_closed(self):
        """
        Closed topics
        """
        topic_ = utils.create_topic(self.category, is_closed=True)
        utils.login(self)
        response = self.client.get(reverse('spirit:admin-topic-closed'))
        self.assertQuerysetEqual(response.context['topics'], map(repr, [topic_, ]))

    def test_topic_pinned(self):
        """
        Pinned topics
        """
        topic_ = utils.create_topic(self.category, is_pinned=True)
        utils.login(self)
        response = self.client.get(reverse('spirit:admin-topic-pinned'))
        self.assertQuerysetEqual(response.context['topics'], map(repr, [topic_, ]))

    def test_category_list(self):
        """
        Categories, excludes Topic Private and subcats
        """
        subcat = utils.create_category(parent=self.category)
        categories = Category.objects.filter(is_private=False, parent=None)
        utils.login(self)
        response = self.client.get(reverse('spirit:admin-category-list'))
        self.assertQuerysetEqual(response.context['categories'], map(repr, categories))

    def test_category_create(self):
        """
        Category create
        """
        utils.login(self)
        form_data = {"parent": "", "title": "foo", "description": "", "is_closed": False, "is_removed": False}
        response = self.client.post(reverse('spirit:admin-category-create'),
                                    form_data)
        expected_url = reverse("spirit:admin-category-list")
        self.assertRedirects(response, expected_url, status_code=302)

        response = self.client.get(reverse('spirit:admin-category-create'))
        self.assertEqual(response.status_code, 200)

    def test_category_update(self):
        """
        Category update
        """
        utils.login(self)
        form_data = {"parent": "", "title": "foo", "description": "", "is_closed": False, "is_removed": False}
        response = self.client.post(reverse('spirit:admin-category-update', kwargs={"category_id": self.category.pk, }),
                                    form_data)
        expected_url = reverse("spirit:admin-category-list")
        self.assertRedirects(response, expected_url, status_code=302)

        response = self.client.get(reverse('spirit:admin-category-update', kwargs={"category_id": self.category.pk, }))
        self.assertEqual(response.status_code, 200)

    def test_config_basic(self):
        """
        Config
        """
        utils.login(self)
        form_data = {"site_name": "foo", "site_description": "bar"}
        response = self.client.post(reverse('spirit:admin-config-basic'),
                                    form_data)
        expected_url = reverse('spirit:admin-config-basic')
        self.assertRedirects(response, expected_url, status_code=302)

        response = self.client.get(reverse('spirit:admin-config-basic'))
        self.assertEqual(response.status_code, 200)

    def test_flag_open(self):
        """
        Open flags
        """
        comment = utils.create_comment(topic=self.topic)
        comment2 = utils.create_comment(topic=self.topic)
        flag_closed = CommentFlag.objects.create(comment=comment2, is_closed=True)
        flag_ = CommentFlag.objects.create(comment=comment)

        utils.login(self)
        response = self.client.get(reverse('spirit:admin-flag-open'))
        self.assertQuerysetEqual(response.context['flags'], map(repr, [flag_, ]))

    def test_flag_closed(self):
        """
        Open flags
        """
        comment = utils.create_comment(topic=self.topic)
        comment2 = utils.create_comment(topic=self.topic)
        flag_closed = CommentFlag.objects.create(comment=comment2, is_closed=True)
        flag_ = CommentFlag.objects.create(comment=comment)

        utils.login(self)
        response = self.client.get(reverse('spirit:admin-flag-closed'))
        self.assertQuerysetEqual(response.context['flags'], map(repr, [flag_closed, ]))

    def test_flag_detail(self):
        """
        flag detail
        """
        comment = utils.create_comment(topic=self.topic)
        comment_flag = CommentFlag.objects.create(comment=comment)
        flag_ = Flag.objects.create(comment=comment, user=self.user, reason=0)

        comment2 = utils.create_comment(topic=self.topic)
        comment_flag2 = CommentFlag.objects.create(comment=comment2)
        flag_2 = Flag.objects.create(comment=comment2, user=self.user, reason=0)

        utils.login(self)
        form_data = {"is_closed": True, }
        response = self.client.post(reverse('spirit:admin-flag-detail', kwargs={'pk': comment_flag.pk, }),
                                    form_data)
        expected_url = reverse('spirit:admin-flag')
        self.assertRedirects(response, expected_url, status_code=302)

        response = self.client.get(reverse('spirit:admin-flag-detail', kwargs={'pk': comment_flag.pk, }))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(repr(response.context['flag']), repr(comment_flag))
        self.assertQuerysetEqual(response.context['flags'], map(repr, [flag_, ]))


class AdminFormTest(TestCase):

    fixtures = ['spirit_init.json', ]

    def setUp(self):
        cache.clear()
        self.user = utils.create_user()
        self.category = utils.create_category()
        self.topic = utils.create_topic(self.category)

    def test_user_edit(self):
        """
        Edit user profile
        """
        form_data = {"username": "fooedit",
                     "email": "foo@bar.com",
                     "location": "Bs As",
                     "timezone": "UTC",
                     "is_administrator": True,
                     "is_moderator": True,
                     "is_active": True}
        form = UserEditForm(data=form_data)
        self.assertEqual(form.is_valid(), True)

    def test_category(self):
        """
        Add category
        """
        form_data = {"parent": "",
                     "title": "foo",
                     "description": "",
                     "is_closed": False,
                     "is_removed": False}
        form = CategoryForm(data=form_data)
        self.assertEqual(form.is_valid(), True)

    def test_category_invalid_parent(self):
        """
        invalid parent
        """
        # parent can not be a subcategory, only one level subcat is allowed
        subcategory = utils.create_category(parent=self.category)
        form_data = {"parent": subcategory.pk, }
        form = CategoryForm(data=form_data)
        self.assertEqual(form.is_valid(), False)
        self.assertNotIn('parent', form.cleaned_data)

        # parent can not be set to a category with childrens
        category_ = utils.create_category()
        form_data = {"parent": category_.pk, }
        form = CategoryForm(data=form_data, instance=self.category)
        self.assertEqual(form.is_valid(), False)
        self.assertNotIn('parent', form.cleaned_data)

        # parent can not be removed
        category_ = utils.create_category(is_removed=True)
        form_data = {"parent": category_.pk, }
        form = CategoryForm(data=form_data)
        self.assertEqual(form.is_valid(), False)
        self.assertNotIn('parent', form.cleaned_data)

        # parent can not be private
        category_ = utils.create_category(is_private=True)
        form_data = {"parent": category_.pk, }
        form = CategoryForm(data=form_data)
        self.assertEqual(form.is_valid(), False)
        self.assertNotIn('parent', form.cleaned_data)

    def test_basic_config(self):
        """
        basic conf edition
        """
        form_data = {"site_name": "foo",
                     "site_description": "",
                     "template_footer": ""}
        form = BasicConfigForm(data=form_data)
        self.assertEqual(form.is_valid(), True)

        form_data = {"site_name": "foo",
                     "site_description": "bar",
                     "template_footer": "foobar"}
        form = BasicConfigForm(data=form_data)
        self.assertEqual(form.is_valid(), True)

    def test_flag(self):
        """
        flag
        """
        comment = utils.create_comment(topic=self.topic)
        comment_flag = CommentFlag.objects.create(comment=comment)

        form_data = {"is_closed": True, }
        form = CommentFlagForm(user=self.user, data=form_data, instance=comment_flag)
        self.assertEqual(form.is_valid(), True)
        self.assertEqual(repr(form.save().moderator), repr(self.user))


#-*- coding: utf-8 -*-

from django.conf.urls import patterns, url
from django.contrib.auth.decorators import login_required

from ..views.search import SearchView
from ..forms.search import AdvancedSearchForm


urlpatterns = patterns("",
    url(r'^$', login_required(SearchView(
        template='spirit/search/search.html',
        form_class=AdvancedSearchForm
    )), name='search'),
)


#-*- coding: utf-8 -*-

import re

from django.db import models
from django.core.urlresolvers import reverse
from django.contrib.auth.models import AbstractBaseUser, PermissionsMixin, UserManager
from django.core.mail import send_mail
from django.core import validators
from django.utils.translation import ugettext_lazy as _
from django.utils import timezone

from spirit.utils.timezone import TIMEZONE_CHOICES
from spirit.utils.models import AutoSlugField


class AbstractForumUser(models.Model):

    slug = AutoSlugField(populate_from="username", db_index=False, blank=True)
    location = models.CharField(_("location"), max_length=75, blank=True)
    last_seen = models.DateTimeField(_("last seen"), auto_now=True)
    last_ip = models.GenericIPAddressField(_("last ip"), blank=True, null=True)
    timezone = models.CharField(_("time zone"), max_length=32, choices=TIMEZONE_CHOICES, default='UTC')
    is_administrator = models.BooleanField(_('administrator status'), default=False)
    is_moderator = models.BooleanField(_('moderator status'), default=False)
    #is_verified = models.BooleanField(_('verified'), default=False)

    topic_count = models.PositiveIntegerField(_("topic count"), default=0)
    comment_count = models.PositiveIntegerField(_("comment count"), default=0)

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        if self.is_superuser:
            self.is_administrator = True

        if self.is_administrator:
            self.is_moderator = True

        super(AbstractForumUser, self).save(*args, **kwargs)


class AbstractUser(AbstractBaseUser, PermissionsMixin, AbstractForumUser):
    # almost verbatim copy from the auth user model
    # adds username(db_index=True), email(unique=True, blank=False, max_length=254)
    username = models.CharField(_("username"), max_length=30, unique=True, db_index=True,
                                help_text=_('Required. 30 characters or fewer. Letters, numbers and '
                                            '@/./+/-/_ characters'),
                                validators=[
                                    validators.RegexValidator(re.compile('^[\w.@+-]+$'), _('Enter a valid username.'),
                                                              'invalid')
                                ])
    first_name = models.CharField(_("first name"), max_length=30, blank=True)
    last_name = models.CharField(_("last name"), max_length=30, blank=True)
    email = models.EmailField(_("email"), max_length=254, unique=True)
    is_staff = models.BooleanField(_('staff status'), default=False,
                                   help_text=_('Designates whether the user can log into this admin '
                                               'site.'))
    is_active = models.BooleanField(_('active'), default=True,
                                    help_text=_('Designates whether this user should be treated as '
                                                'active. Unselect this instead of deleting accounts.'))
    date_joined = models.DateTimeField(_('date joined'), default=timezone.now)

    objects = UserManager()

    USERNAME_FIELD = 'username'
    REQUIRED_FIELDS = ['email', ]

    class Meta:
        abstract = True

    def get_absolute_url(self):
        return reverse('spirit:profile-detail', kwargs={'pk': self.pk,
                                                        'slug': self.slug})

    def get_full_name(self):
        full_name = '%s %s' % (self.first_name, self.last_name)
        return full_name.strip()

    def get_short_name(self):
        return self.first_name

    def email_user(self, subject, message, from_email=None):
        send_mail(subject, message, from_email, [self.email, ])


class User(AbstractUser):

    class Meta(AbstractUser.Meta):
        swappable = 'AUTH_USER_MODEL'
        app_label = 'spirit'
        ordering = ['-date_joined', ]
        verbose_name = _('user')
        verbose_name_plural = _('users')


#-*- coding: utf-8 -*-

from django.db.models import Manager
from django.shortcuts import get_object_or_404
from django.db.models import Q


class CommentManager(Manager):

    def _for_all(self):
        return self.filter(Q(topic__category__parent=None) | Q(topic__category__parent__is_removed=False),
                           topic__category__is_removed=False,
                           topic__is_removed=False,
                           is_removed=False,
                           action=0)\
            .select_related('user')

    def for_all(self):
        return self.filter(is_removed=False, action=0)\
            .select_related('user')

    def for_topic(self, topic):
        return self.filter(topic=topic)\
            .select_related('user')

    def for_public(self):
        return self._for_all()\
            .filter(topic__category__is_private=False)

    def for_user_public(self, user):
        return self.for_public()\
            .filter(user=user)

    def for_access(self, user):
        return self._for_all()\
            .filter(Q(topic__category__is_private=False) | Q(topic__topics_private__user=user))

    def for_update_or_404(self, pk, user):
        if user.is_moderator:
            return get_object_or_404(self, pk=pk)
        else:
            return get_object_or_404(self.for_access(user), user=user, pk=pk)


#-*- coding: utf-8 -*-

import hashlib
import os

from markdown import Markdown

from django import forms
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.utils.image import Image

from spirit.models.comment import Comment
from spirit.models.topic import Topic
from spirit import utils


class CommentForm(forms.ModelForm):

    class Meta:
        model = Comment
        fields = ['comment', ]

    def __init__(self, user=None, topic=None, *args, **kwargs):
        super(CommentForm, self).__init__(*args, **kwargs)
        self.user = user
        self.topic = topic
        self.mentions = None  # {username: User, }
        self.fields['comment'].widget.attrs['placeholder'] = _("Write comment...")

    def _get_comment_html(self):
        markdown = Markdown(output_formats='html5',
                            safe_mode='escape',
                            extensions=settings.ST_MARKDOWN_EXT)
        markdown.mentions = {}
        comment_html = markdown.convert(self.cleaned_data['comment'])
        self.mentions = markdown.mentions
        return comment_html

    def save(self, commit=True):
        if not self.instance.pk:
            self.instance.user = self.user
            self.instance.topic = self.topic

        self.instance.comment_html = self._get_comment_html()
        return super(CommentForm, self).save(commit)


class CommentMoveForm(forms.Form):

    topic = forms.ModelChoiceField(queryset=Topic.objects.all(), widget=forms.TextInput)

    def __init__(self, topic, *args, **kwargs):
        super(CommentMoveForm, self).__init__(*args, **kwargs)
        self.fields['comments'] = forms.ModelMultipleChoiceField(queryset=Comment.objects.filter(topic=topic),
                                                                 widget=forms.CheckboxSelectMultiple)

    def save(self):
        comments = self.cleaned_data['comments']
        comments_list = list(comments)
        topic = self.cleaned_data['topic']
        comments.update(topic=topic)

        # Update topic in comment instance
        for c in comments_list:
            c.topic = topic

        return comments_list


class CommentImageForm(forms.Form):

    image = forms.ImageField()

    def __init__(self, user=None, *args, **kwargs):
        super(CommentImageForm, self).__init__(*args, **kwargs)
        self.user = user

    def clean_image(self):
        image = self.cleaned_data["image"]
        image.format = Image.open(image).format.lower()
        image.seek(0)

        if image.format not in settings.ST_ALLOWED_UPLOAD_IMAGE_FORMAT:
            raise forms.ValidationError(_("Unsupported file format. Supported formats are %s."
                                          % ", ".join(settings.ST_ALLOWED_UPLOAD_IMAGE_FORMAT)))

        return image

    def save(self):
        image = self.cleaned_data["image"]
        hash = hashlib.md5(image.read()).hexdigest()
        image.name = u"".join((hash, ".", image.format))
        upload_to = os.path.join('spirit', 'images', str(self.user.pk))
        image.url = os.path.join(settings.MEDIA_URL, upload_to, image.name).replace("\\", "/")
        media_path = os.path.join(settings.MEDIA_ROOT, upload_to)
        utils.mkdir_p(media_path)

        with open(os.path.join(media_path, image.name), "wb") as fh:
            image.seek(0)
            fh.write(image.read())
            image.close()

        return image


#-*- coding: utf-8 -*-

from django.conf.urls import patterns, url


urlpatterns = patterns("spirit.views.topic_private",
    url(r'^$', 'private_list', name='private-list'),
    url(r'^created/$', 'private_created_list', name='private-created-list'),

    url(r'^publish/$', 'private_publish', name='private-publish'),
    url(r'^publish/(?P<user_id>\d+)/$', 'private_publish', name='private-publish'),

    url(r'^(?P<topic_id>\d+)/$', 'private_detail', kwargs={'slug': "", }, name='private-detail'),
    url(r'^(?P<topic_id>\d+)/(?P<slug>[\w-]+)/$', 'private_detail', name='private-detail'),

    url(r'^invite/(?P<topic_id>\d+)/$', 'private_access_create', name='private-access-create'),
    url(r'^remove/(?P<pk>\d+)/$', 'private_access_delete', name='private-access-remove'),
    url(r'^join/(?P<topic_id>\d+)/$', 'private_join', name='private-join'),
)


#-*- coding: utf-8 -*-

from django.contrib.auth.decorators import login_required
from django.shortcuts import render, redirect, get_object_or_404

from spirit.models.comment import Comment

from spirit.forms.comment_flag import FlagForm


@login_required
def flag_create(request, comment_id):
    comment = get_object_or_404(Comment, pk=comment_id)

    if request.method == 'POST':
        form = FlagForm(user=request.user, comment=comment, data=request.POST)

        if form.is_valid():
            form.save()
            return redirect(request.POST.get('next', comment.get_absolute_url()))
    else:
        form = FlagForm()

    return render(request, 'spirit/comment_flag/flag_create.html', {'form': form, 'comment': comment})


#-*- coding: utf-8 -*-

from django.contrib.auth.backends import ModelBackend

from django.contrib.auth import get_user_model


User = get_user_model()


class EmailAuthBackend(ModelBackend):

    def authenticate(self, username=None, password=None, **kwargs):
        try:
            user = User._default_manager.get(email=username)

            if user.check_password(password):
                return user
        except User.DoesNotExist:
            pass


#-*- coding: utf-8 -*-

from django.contrib.auth.decorators import login_required
from django.shortcuts import render, redirect, get_object_or_404
from django.core.urlresolvers import reverse

from spirit.models.comment import Comment

from spirit.models.comment_like import CommentLike
from spirit.forms.comment_like import LikeForm
from ..signals.comment_like import comment_like_post_create, comment_like_post_delete
from spirit.utils import json_response


@login_required
def like_create(request, comment_id):
    comment = get_object_or_404(Comment.objects.exclude(user=request.user), pk=comment_id)

    if request.method == 'POST':
        form = LikeForm(user=request.user, comment=comment, data=request.POST)

        if form.is_valid():
            like = form.save()
            comment_like_post_create.send(sender=like.__class__, comment=like.comment)

            if request.is_ajax():
                return json_response({'url_delete': like.get_delete_url(), })

            return redirect(request.POST.get('next', comment.get_absolute_url()))
    else:
        form = LikeForm()

    return render(request, 'spirit/comment_like/like_create.html', {'form': form, 'comment': comment})


@login_required
def like_delete(request, pk):
    like = get_object_or_404(CommentLike, pk=pk, user=request.user)

    if request.method == 'POST':
        like.delete()
        comment_like_post_delete.send(sender=like.__class__, comment=like.comment)

        if request.is_ajax():
            return json_response({'url_create': reverse('spirit:like-create',
                                                        kwargs={'comment_id': like.comment.pk, }), })

        return redirect(request.POST.get('next', like.comment.get_absolute_url()))

    return render(request, 'spirit/comment_like/like_delete.html', {'like': like, })


#-*- coding: utf-8 -*-

from django.contrib.auth.decorators import login_required
from django.core.urlresolvers import reverse
from django.contrib.auth import get_user_model
from django.shortcuts import render, redirect, get_object_or_404
from django.views.decorators.http import require_POST
from django.contrib import messages
from django.http import HttpResponsePermanentRedirect
from django.utils.translation import ugettext as _
from django.conf import settings

from spirit.utils.ratelimit.decorators import ratelimit
from spirit import utils
from spirit.forms.comment import CommentForm
from spirit.signals.comment import comment_posted

from spirit.models.topic import Topic
from spirit.signals.topic import topic_viewed
from ..models.topic_private import TopicPrivate
from ..forms.topic_private import TopicPrivateManyForm, TopicForPrivateForm,\
    TopicPrivateJoinForm, TopicPrivateInviteForm
from spirit.signals.topic_private import topic_private_post_create, topic_private_access_pre_create


User = get_user_model()


@login_required
@ratelimit(rate='1/10s')
def private_publish(request, user_id=None):
    if request.method == 'POST':
        tform = TopicForPrivateForm(user=request.user, data=request.POST)
        cform = CommentForm(user=request.user, data=request.POST)
        tpform = TopicPrivateManyForm(user=request.user, data=request.POST)

        if not request.is_limited and tform.is_valid() and cform.is_valid() and tpform.is_valid():
            # wrap in transaction.atomic?
            topic = tform.save()
            cform.topic = topic
            comment = cform.save()
            comment_posted.send(sender=comment.__class__, comment=comment, mentions=None)
            tpform.topic = topic
            topics_private = tpform.save_m2m()
            topic_private_post_create.send(sender=TopicPrivate, topics_private=topics_private, comment=comment)
            return redirect(topic.get_absolute_url())

    else:
        tform = TopicForPrivateForm()
        cform = CommentForm()
        initial = None

        if user_id:
            user = get_object_or_404(User, pk=user_id)
            initial = {'users': [user.username, ]}

        tpform = TopicPrivateManyForm(initial=initial)

    return render(request, 'spirit/topic_private/private_publish.html', {'tform': tform,
                                                                         'cform': cform,
                                                                         'tpform': tpform})


@login_required
def private_detail(request, topic_id, slug):
    topic_private = get_object_or_404(TopicPrivate.objects.select_related('topic'),
                                      topic_id=topic_id, user=request.user)

    if topic_private.topic.slug != slug:
        return HttpResponsePermanentRedirect(topic_private.get_absolute_url())

    topic_viewed.send(sender=topic_private.topic.__class__, request=request, topic=topic_private.topic)

    return render(request, 'spirit/topic_private/private_detail.html', {'topic': topic_private.topic,
                                                                        'topic_private': topic_private,
                                                                        'COMMENTS_PER_PAGE': settings.ST_COMMENTS_PER_PAGE})


@login_required
@require_POST
def private_access_create(request, topic_id):
    topic_private = TopicPrivate.objects.for_create_or_404(topic_id, request.user)
    form = TopicPrivateInviteForm(topic=topic_private.topic, data=request.POST)

    if form.is_valid():
        topic_private_access_pre_create.send(sender=topic_private.__class__,
                                             topic=topic_private.topic,
                                             user=form.cleaned_data['user'])
        form.save()
    else:
        messages.error(request, utils.render_form_errors(form))

    return redirect(request.POST.get('next', topic_private.get_absolute_url()))


@login_required
def private_access_delete(request, pk):
    topic_private = TopicPrivate.objects.for_delete_or_404(pk, request.user)

    if request.method == 'POST':
        topic_private.delete()

        if request.user.pk == topic_private.user_id:
            return redirect(reverse("spirit:private-list"))

        return redirect(request.POST.get('next', topic_private.get_absolute_url()))
    else:
        return render(request, 'spirit/topic_private/private_delete.html', {'topic_private': topic_private, })


@login_required
def private_join(request, topic_id):
    # This is for topic creators who left their own topics and want to join again
    topic = get_object_or_404(Topic, pk=topic_id, user=request.user, category_id=settings.ST_TOPIC_PRIVATE_CATEGORY_PK)

    if request.method == 'POST':
        form = TopicPrivateJoinForm(topic=topic, user=request.user, data=request.POST)

        if form.is_valid():
            topic_private_access_pre_create.send(sender=TopicPrivate, topic=topic, user=request.user)
            form.save()

            return redirect(request.POST.get('next', topic.get_absolute_url()))
    else:
        form = TopicPrivateJoinForm()

    return render(request, 'spirit/topic_private/private_join.html', {'topic': topic, 'form': form, })


@login_required
def private_list(request):
    topics = Topic.objects.filter(topics_private__user=request.user).order_by('-last_active')
    return render(request, 'spirit/topic_private/private_list.html', {'topics': topics, })


@login_required
def private_created_list(request):
    # Show created topics but exclude those the user is participating on
    # TODO: show all, show join link in those the user is not participating
    topics = Topic.objects.filter(user=request.user, category_id=settings.ST_TOPIC_PRIVATE_CATEGORY_PK)\
        .exclude(topics_private__user=request.user)
    return render(request, 'spirit/topic_private/private_created_list.html', {'topics': topics, })


#-*- coding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse
from django.conf import settings
from django.utils import timezone
from django.db import IntegrityError

from spirit.signals.comment import comment_posted
from spirit.signals.topic_private import topic_private_post_create, topic_private_access_pre_create
from spirit.signals.topic import topic_viewed

from spirit.managers.topic_notifications import TopicNotificationManager


UNDEFINED, MENTION, COMMENT = xrange(3)

ACTION_CHOICES = (
    (UNDEFINED, _("Undefined")),
    (MENTION, _("Mention")),
    (COMMENT, _("Comment")),
)


class TopicNotification(models.Model):

    user = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name=_("user"))
    topic = models.ForeignKey('spirit.Topic')
    comment = models.ForeignKey('spirit.Comment', null=True, blank=True)

    date = models.DateTimeField(auto_now_add=True)
    action = models.IntegerField(choices=ACTION_CHOICES, default=UNDEFINED)
    is_read = models.BooleanField(default=False)
    is_active = models.BooleanField(default=False)

    objects = TopicNotificationManager()

    class Meta:
        app_label = 'spirit'
        unique_together = ('user', 'topic')
        ordering = ['-date', ]
        verbose_name = _("topic notification")
        verbose_name_plural = _("topics notification")

    def get_absolute_url(self):
        return self.comment.get_absolute_url()

    @property
    def text_action(self):
        return ACTION_CHOICES[self.action][1]

    @property
    def is_mention(self):
        return self.action == MENTION

    @property
    def is_comment(self):
        return self.action == COMMENT

    def __unicode__(self):
        return "%s in %s" % (self.user, self.topic)


def notification_comment_posted_handler(sender, comment, **kwargs):
    # Create Notification for poster
    # if not exists create a dummy one with defaults
    try:
        TopicNotification.objects.get_or_create(user=comment.user, topic=comment.topic,
                                                defaults={'action': COMMENT,
                                                          'is_read': True,
                                                          'is_active': True})
    except IntegrityError:
        pass

    TopicNotification.objects.filter(topic=comment.topic, is_active=True, is_read=True)\
        .exclude(user=comment.user)\
        .update(comment=comment, is_read=False, action=COMMENT, date=timezone.now())


def mention_comment_posted_handler(sender, comment, mentions, **kwargs):
    if not mentions:
        return

    for username, user in mentions.iteritems():
        try:
            TopicNotification.objects.create(user=user, topic=comment.topic,
                                             comment=comment, action=MENTION)
        except IntegrityError:
            pass

    TopicNotification.objects.filter(user__in=mentions.values(), topic=comment.topic, is_read=True)\
        .update(comment=comment, is_read=False, action=MENTION, date=timezone.now())


def comment_posted_handler(sender, comment, mentions, **kwargs):
    notification_comment_posted_handler(sender, comment, **kwargs)
    mention_comment_posted_handler(sender, comment, mentions, **kwargs)


def topic_private_post_create_handler(sender, topics_private, comment, **kwargs):
    # topic.user notification is created on comment_posted
    TopicNotification.objects.bulk_create([TopicNotification(user=tp.user, topic=tp.topic,
                                                             comment=comment, action=COMMENT,
                                                             is_active=True)
                                           for tp in topics_private
                                           if tp.user != tp.topic.user])


def topic_private_access_pre_create_handler(sender, topic, user, **kwargs):
    # TODO: use update_or_create on django 1.7
    # change to post create
    try:
        TopicNotification.objects.create(user=user, topic=topic,
                                         comment=topic.comment_set.last(), action=COMMENT,
                                         is_active=True)
    except IntegrityError:
        pass


def topic_viewed_handler(sender, request, topic, **kwargs):
    if not request.user.is_authenticated():
        return

    TopicNotification.objects.filter(user=request.user, topic=topic)\
        .update(is_read=True)


comment_posted.connect(comment_posted_handler, dispatch_uid=__name__)
topic_private_post_create.connect(topic_private_post_create_handler, dispatch_uid=__name__)
topic_private_access_pre_create.connect(topic_private_access_pre_create_handler, dispatch_uid=__name__)
topic_viewed.connect(topic_viewed_handler, dispatch_uid=__name__)


#-*- coding: utf-8 -*-

import datetime

from django.test import TestCase
from django.core.cache import cache
from django.core.urlresolvers import reverse
from django.template import Template, Context
from django.conf import settings
from django.utils import timezone

import utils

from spirit.models.category import Category
from spirit.models.topic_private import TopicPrivate
from spirit.forms.topic_private import TopicForPrivateForm, TopicPrivateInviteForm,\
    TopicPrivateManyForm, TopicPrivateJoinForm
from spirit.templatetags.tags.topic_private import render_invite_form
from spirit.views.topic_private import comment_posted
from spirit.models.comment import Comment
from spirit.signals.topic_private import topic_private_post_create, topic_private_access_pre_create
from spirit.models.topic import Topic


class TopicPrivateViewTest(TestCase):

    fixtures = ['spirit_init.json', ]

    def setUp(self):
        cache.clear()
        self.user = utils.create_user()
        self.user2 = utils.create_user()

    def test_private_publish(self):
        """
        POST, create private topic
        """
        utils.login(self)
        form_data = {'comment': 'foo', 'title': 'foobar', 'users': self.user2.username}
        response = self.client.post(reverse('spirit:private-publish'),
                                    form_data)
        private = TopicPrivate.objects.last()
        expected_url = private.get_absolute_url()
        self.assertRedirects(response, expected_url, status_code=302)

        response = self.client.get(reverse('spirit:private-publish'))
        self.assertEqual(response.status_code, 200)

    def test_private_publish_comment_posted_signals(self):
        """
        send publish_comment_posted signal
        """
        def comment_posted_handler(sender, comment, **kwargs):
            self._comment = repr(comment)
        comment_posted.connect(comment_posted_handler)

        utils.login(self)
        form_data = {'comment': 'foo', 'title': 'foobar', 'users': self.user2.username}
        response = self.client.post(reverse('spirit:private-publish'),
                                    form_data)
        self.assertEqual(response.status_code, 302)
        comment = Comment.objects.last()
        self.assertEqual(self._comment, repr(comment))

    def test_private_publish_topic_private_post_create_signals(self):
        """
        send topic_private_post_create signal
        """
        def topic_private_post_create_handler(sender, topics_private, comment, **kwargs):
            self.assertEqual(len(topics_private), 1)
            tp = topics_private[0]
            self._topic = repr(tp.topic)
            self._user = repr(tp.user)
            self._comment = repr(comment)
        topic_private_post_create.connect(topic_private_post_create_handler)

        utils.login(self)
        form_data = {'comment': 'foo', 'title': 'foobar', 'users': self.user.username}
        response = self.client.post(reverse('spirit:private-publish'),
                                    form_data)
        self.assertEqual(response.status_code, 302)
        topic_private = TopicPrivate.objects.last()
        topic_comment = Comment.objects.last()
        self.assertEqual(self._topic, repr(topic_private.topic))
        self.assertEqual(self._user, repr(self.user))
        self.assertEqual(self._comment, repr(topic_comment))

    def test_private_publish_user(self):
        """
        create private topic with user as initial value
        """
        utils.login(self)
        response = self.client.get(reverse('spirit:private-publish', kwargs={'user_id': self.user2.pk, }))
        self.assertEqual(response.context['tpform'].initial['users'], [self.user2.username, ])

    def test_private_detail(self):
        """
        private topic detail
        """
        utils.login(self)
        private = utils.create_private_topic(user=self.user)
        response = self.client.get(reverse('spirit:private-detail', kwargs={'topic_id': private.topic.pk,
                                                                            'slug': private.topic.slug}))
        self.assertEqual(response.context['topic'], private.topic)

    def test_private_access_create(self):
        """
        private topic access creation
        """
        utils.login(self)
        private = utils.create_private_topic(user=self.user)
        form_data = {'user': self.user2.username, }
        response = self.client.post(reverse('spirit:private-access-create', kwargs={'topic_id': private.topic.pk, }),
                                    form_data)
        expected_url = private.get_absolute_url()
        self.assertRedirects(response, expected_url, status_code=302)
        self.assertEqual(len(TopicPrivate.objects.filter(user=self.user2, topic=private.topic)), 1)

    def test_private_access_create_invalid(self):
        """
        Only the topic owner should be able to invite
        """
        utils.login(self)
        private = utils.create_private_topic(user=self.user2)
        TopicPrivate.objects.create(user=self.user, topic=private.topic)
        user = utils.create_user()
        form_data = {'user': user.username, }
        response = self.client.post(reverse('spirit:private-access-create', kwargs={'topic_id': private.topic.pk, }),
                                    form_data)
        self.assertEqual(response.status_code, 404)

    def test_private_access_create_pre_create_signal(self):
        """
        send topic_private_access_pre_create signal
        """
        def topic_private_access_pre_create_handler(sender, topic, user, **kwargs):
            self._topic = repr(topic)
            self._user = repr(user)
        topic_private_access_pre_create.connect(topic_private_access_pre_create_handler)

        utils.login(self)
        private = utils.create_private_topic(user=self.user)
        form_data = {'user': self.user2.username, }
        response = self.client.post(reverse('spirit:private-access-create', kwargs={'topic_id': private.topic.pk, }),
                                    form_data)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(self._topic, repr(private.topic))
        self.assertEqual(self._user, repr(self.user2))

    def test_private_access_delete(self):
        """
        private topic access deletion
        """
        utils.login(self)
        private = utils.create_private_topic(user=self.user)
        private2 = TopicPrivate.objects.create(user=self.user2, topic=private.topic)
        form_data = {}
        response = self.client.post(reverse('spirit:private-access-remove', kwargs={'pk': private2.pk, }),
                                    form_data)
        expected_url = private.get_absolute_url()
        self.assertRedirects(response, expected_url, status_code=302)

    def test_private_access_delete_invalid(self):
        """
        Only the topic owner should be able to remove accesses
        """
        utils.login(self)
        private = utils.create_private_topic(user=self.user2)
        TopicPrivate.objects.create(user=self.user, topic=private.topic)
        form_data = {}
        response = self.client.post(reverse('spirit:private-access-remove', kwargs={'pk': private.pk, }),
                                    form_data)
        self.assertEqual(response.status_code, 404)

    def test_private_access_delete_leave(self):
        """
        user should be able to remove himself
        """
        utils.login(self)
        private = utils.create_private_topic(user=self.user2)
        private2_leave = TopicPrivate.objects.create(user=self.user, topic=private.topic)
        form_data = {}
        response = self.client.post(reverse('spirit:private-access-remove', kwargs={'pk': private2_leave.pk, }),
                                    form_data)
        expected_url = reverse("spirit:private-list")
        self.assertRedirects(response, expected_url, status_code=302)

    def test_private_list(self):
        """
        private topic list
        """
        private = utils.create_private_topic(user=self.user)
        # dont show private topics from other users
        private2 = TopicPrivate.objects.create(user=self.user2, topic=private.topic)
        # dont show topics from other categories
        category = utils.create_category()
        topic = utils.create_topic(category, user=self.user)

        utils.login(self)
        response = self.client.get(reverse('spirit:private-list'))
        self.assertQuerysetEqual(response.context['topics'], [repr(private.topic), ])

    def test_private_list_order_topics(self):
        """
        private topic list ordered by last active
        """
        private_a = utils.create_private_topic(user=self.user)
        private_b = utils.create_private_topic(user=self.user)
        private_c = utils.create_private_topic(user=self.user)

        Topic.objects.filter(pk=private_a.topic.pk).update(last_active=timezone.now() - datetime.timedelta(days=10))
        Topic.objects.filter(pk=private_c.topic.pk).update(last_active=timezone.now() - datetime.timedelta(days=5))

        utils.login(self)
        response = self.client.get(reverse('spirit:private-list'))
        self.assertQuerysetEqual(response.context['topics'], map(repr, [private_b.topic, private_c.topic,
                                                                        private_a.topic]))

    def test_private_join(self):
        """
        private topic join
        """
        private = utils.create_private_topic(user=self.user)
        private.delete()

        utils.login(self)
        form_data = {}
        response = self.client.post(reverse('spirit:private-join', kwargs={'topic_id': private.topic.pk, }),
                                    form_data)
        expected_url = private.get_absolute_url()
        self.assertRedirects(response, expected_url, status_code=302)

        response = self.client.get(reverse('spirit:private-join', kwargs={'topic_id': private.topic.pk, }))
        self.assertEqual(response.status_code, 200)

    def test_private_join_invalid_regular_topic(self):
        """
        Only topics from the category private can be rejoined
        """
        category = utils.create_category()
        topic = utils.create_topic(category, user=self.user)

        utils.login(self)
        form_data = {}
        response = self.client.post(reverse('spirit:private-join', kwargs={'topic_id': topic.pk, }),
                                    form_data)
        self.assertEqual(response.status_code, 404)

    def test_private_join_invalid_not_owner(self):
        """
        Only topic creators/owners can rejoin
        """
        private = utils.create_private_topic(user=self.user2)

        utils.login(self)
        form_data = {}
        response = self.client.post(reverse('spirit:private-join', kwargs={'topic_id': private.topic.pk, }),
                                    form_data)
        self.assertEqual(response.status_code, 404)

    def test_private_join_access_pre_create_signal(self):
        """
        send topic_private_access_pre_create signal
        """
        def topic_private_access_pre_create_handler(sender, topic, user, **kwargs):
            self._topic = repr(topic)
            self._user_pk = user.pk
        topic_private_access_pre_create.connect(topic_private_access_pre_create_handler)

        private = utils.create_private_topic(user=self.user)
        private.delete()

        utils.login(self)
        form_data = {}
        response = self.client.post(reverse('spirit:private-join', kwargs={'topic_id': private.topic.pk, }),
                                    form_data)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(self._topic, repr(private.topic))
        self.assertEqual(self._user_pk, self.user.pk)

    def test_private_created_list(self):
        """
        private topic created list, shows only the private topics the user is no longer participating
        """
        category = utils.create_category()
        regular_topic = utils.create_topic(category, user=self.user)
        # it's the owner, left the topic
        private = utils.create_private_topic(user=self.user)
        private.delete()
        # has access and is the owner
        private2 = utils.create_private_topic(user=self.user)
        # does not has access
        private3 = utils.create_private_topic(user=self.user2)
        # has access but it's not owner
        private4 = utils.create_private_topic(user=self.user2)
        TopicPrivate.objects.create(user=self.user, topic=private4.topic)

        utils.login(self)
        response = self.client.get(reverse('spirit:private-created-list'))
        self.assertQuerysetEqual(response.context['topics'], map(repr, [private.topic, ]))

    def test_private_created_list_order_topics(self):
        """
        private topic created list ordered by last active
        """
        private_a = utils.create_private_topic(user=self.user)
        private_b = utils.create_private_topic(user=self.user)
        private_c = utils.create_private_topic(user=self.user)
        private_a.delete()
        private_b.delete()
        private_c.delete()

        Topic.objects.filter(pk=private_a.topic.pk).update(last_active=timezone.now() - datetime.timedelta(days=10))
        Topic.objects.filter(pk=private_c.topic.pk).update(last_active=timezone.now() - datetime.timedelta(days=5))

        utils.login(self)
        response = self.client.get(reverse('spirit:private-created-list'))
        self.assertQuerysetEqual(response.context['topics'], map(repr, [private_b.topic, private_c.topic,
                                                                        private_a.topic]))


class TopicPrivateFormTest(TestCase):

    fixtures = ['spirit_init.json', ]

    def setUp(self):
        cache.clear()
        self.user = utils.create_user()
        self.user2 = utils.create_user()

    def test_private_publish(self):
        """
        create simple topic
        """
        form_data = {'title': 'foo', }
        form = TopicForPrivateForm(data=form_data)
        self.assertEqual(form.is_valid(), True)

    def test_private_create_many(self):
        """
        create many private topics accesses
        """
        users = '%s, %s' % (self.user.username, self.user2.username)
        form_data = {'users': users, }
        form = TopicPrivateManyForm(self.user, data=form_data)
        self.assertEqual(form.is_valid(), True)

        category = Category.objects.get(pk=settings.ST_TOPIC_PRIVATE_CATEGORY_PK)
        topic = utils.create_topic(category=category, user=self.user)
        form.topic = topic
        privates_saved = form.save_m2m()
        privates = TopicPrivate.objects.all()
        self.assertItemsEqual(map(repr, privates_saved), map(repr, privates))

    def test_private_create(self):
        """
        create single private topic access
        """
        category = Category.objects.get(pk=settings.ST_TOPIC_PRIVATE_CATEGORY_PK)
        topic = utils.create_topic(category=category, user=self.user)
        form_data = {'user': self.user.username, }
        form = TopicPrivateInviteForm(data=form_data)
        form.topic = topic
        self.assertEqual(form.is_valid(), True)

    def test_private_join(self):
        """
        re-join private topic if user is the creator
        """
        private = utils.create_private_topic(user=self.user)
        private.delete()

        form_data = {}
        form = TopicPrivateJoinForm(user=self.user, topic=private.topic, data=form_data)
        self.assertTrue(form.is_valid())
        private_topic = form.save()
        self.assertEqual(repr(private_topic.topic), repr(private.topic))
        self.assertEqual(repr(private_topic.user), repr(private.user))

        # topic private exists
        private = utils.create_private_topic(user=self.user)
        form = TopicPrivateJoinForm(user=self.user, topic=private.topic, data=form_data)
        self.assertFalse(form.is_valid())


class TopicTemplateTagsTest(TestCase):

    fixtures = ['spirit_init.json', ]

    def setUp(self):
        cache.clear()
        self.user = utils.create_user()
        self.category = Category.objects.get(pk=settings.ST_TOPIC_PRIVATE_CATEGORY_PK)
        self.topic = utils.create_topic(category=self.category, user=self.user)

    def test_render_invite_form(self):
        """
        display invite form
        """
        out = Template(
            "{% load spirit_tags %}"
            "{% render_invite_form topic %}"
        ).render(Context({'topic': self.topic, }))
        self.assertNotEqual(out, "")
        context = render_invite_form(self.topic)
        self.assertEqual(context['next'], None)
        self.assertIsInstance(context['form'], TopicPrivateInviteForm)
        self.assertEqual(repr(context['topic']), repr(self.topic))


#-*- coding: utf-8 -*-

from django.conf.urls import patterns, url


urlpatterns = patterns("spirit.views.admin.category",
    url(r'^$', 'category_list', name='admin-category'),
    url(r'^list/$', 'category_list', name='admin-category-list'),
    url(r'^create/$', 'category_create', name='admin-category-create'),
    url(r'^update/(?P<category_id>\d+)/$', 'category_update', name='admin-category-update'),
)


#-*- coding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse
from django.conf import settings
from django.db import IntegrityError

from spirit.signals.topic import topic_viewed
from ..utils import paginator


class CommentBookmark(models.Model):

    user = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name=_("user"))
    topic = models.ForeignKey('spirit.Topic')

    comment_number = models.PositiveIntegerField(default=0)

    class Meta:
        app_label = 'spirit'
        unique_together = ('user', 'topic')
        verbose_name = _("comment bookmark")
        verbose_name_plural = _("comments bookmarks")

    def get_absolute_url(self):
        return paginator.get_url(self.topic.get_absolute_url(),
                                 self.comment_number,
                                 settings.ST_COMMENTS_PER_PAGE,
                                 settings.ST_COMMENTS_PAGE_VAR)

    def __unicode__(self):
        return "%s bookmarked comment %s in %s" % (self.user.username,
                                                   self.topic.title,
                                                   self.comment_number)


def topic_page_viewed_handler(sender, request, topic, **kwargs):
    if not request.user.is_authenticated():
        return

    try:
        page_number = int(request.GET.get(settings.ST_COMMENTS_PAGE_VAR, 1))
    except ValueError:
        return

    comment_number = settings.ST_COMMENTS_PER_PAGE * (page_number - 1) + 1

    # TODO: use update_or_create on django 1.7
    try:
        CommentBookmark.objects.create(user=request.user, topic=topic, comment_number=comment_number)
    except IntegrityError:
        CommentBookmark.objects.filter(user=request.user, topic=topic)\
            .update(comment_number=comment_number)


topic_viewed.connect(topic_page_viewed_handler, dispatch_uid=__name__)


#-*- coding: utf-8 -*-

from django.shortcuts import render
from django.contrib.auth.decorators import login_required

from spirit.utils.paginator.infinite_paginator import paginate

from spirit.models.topic import Topic


@login_required
def topic_unread_list(request):
    # TODO: add button to clean up read topics? or read all?
    # redirect to first page if empty

    topics = Topic.objects.for_access(request.user)\
        .filter(topicunread__user=request.user,
                topicunread__is_read=False)

    page = paginate(request, query_set=topics, lookup_field="last_active", page_var='topic_id')
    next_page_pk = None

    if page:
        next_page_pk = page[-1].pk

    return render(request, 'spirit/topic_unread/list.html', {'page': page,
                                                             'next_page_pk': next_page_pk})


#-*- coding: utf-8 -*-

from django import forms
from django.utils.translation import ugettext_lazy as _
from django.db import IntegrityError
from django.utils import timezone

from spirit.models.comment_flag import Flag, CommentFlag


class FlagForm(forms.ModelForm):

    class Meta:
        model = Flag
        fields = ['reason', 'body']

    def __init__(self, user=None, comment=None, *args, **kwargs):
        super(FlagForm, self).__init__(*args, **kwargs)
        self.user = user
        self.comment = comment

    def clean(self):
        cleaned_data = super(FlagForm, self).clean()

        flag = Flag.objects.filter(user=self.user,
                                   comment=self.comment)

        if flag.exists():
            # Do this since some of the unique_together fields are excluded.
            raise forms.ValidationError(_("This flag already exists"))

        return cleaned_data

    def save(self, commit=True):
        if not self.instance.pk:
            self.instance.user = self.user
            self.instance.comment = self.comment

            # TODO: use update_or_create on django 1.7
            try:
                CommentFlag.objects.create(comment=self.comment)
            except IntegrityError:
                CommentFlag.objects.filter(comment=self.comment)\
                    .update(date=timezone.now())

        return super(FlagForm, self).save(commit)


#-*- coding: utf-8 -*-

from django.shortcuts import render, redirect, get_object_or_404
from django.core.urlresolvers import reverse
from django.contrib import messages
from django.utils.translation import ugettext as _

from spirit.utils.decorators import administrator_required

from spirit.models.comment_flag import CommentFlag, Flag
from spirit.forms.admin import CommentFlagForm


@administrator_required
def flag_open(request):
    flags = CommentFlag.objects.filter(is_closed=False)
    return render(request, 'spirit/admin/comment_flag/flag_open.html', {'flags': flags, })


@administrator_required
def flag_closed(request):
    flags = CommentFlag.objects.filter(is_closed=True)
    return render(request, 'spirit/admin/comment_flag/flag_closed.html', {'flags': flags, })


@administrator_required
def flag_detail(request, pk):
    flag = get_object_or_404(CommentFlag, pk=pk)

    if request.method == 'POST':
        form = CommentFlagForm(user=request.user, data=request.POST, instance=flag)

        if form.is_valid():
            form.save()
            messages.info(request, _("The flag has been moderated!"))
            return redirect(reverse("spirit:admin-flag"))
    else:
        form = CommentFlagForm(instance=flag)

    flags = Flag.objects.filter(comment=flag.comment)
    return render(request, 'spirit/admin/comment_flag/flag_detail.html', {'flag': flag,
                                                                          'flags': flags,
                                                                          'form': form})


#-*- coding: utf-8 -*-

from django.http import Http404

from infinite_scroll_pagination.paginator import SeekPaginator, EmptyPage


def paginate(request, query_set, lookup_field, per_page=15, page_var='value'):
    page_pk = request.GET.get(page_var, None)
    paginator = SeekPaginator(query_set, per_page=per_page, lookup_field=lookup_field)

    # First page
    if page_pk is None:
        return paginator.page()

    try:
        obj = query_set.model.objects.get(pk=page_pk)
    except query_set.model.DoesNotExist:
        raise Http404()

    value = getattr(obj, lookup_field)

    try:
        page = paginator.page(value=value, pk=page_pk)
    except EmptyPage:
        raise Http404()

    return page


#-*- coding: utf-8 -*-

from django import forms
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth import get_user_model
from django.conf import settings

from ..models.topic import Topic
from ..models.category import Category

from ..utils.widgets import MultipleInput
from ..models.topic_private import TopicPrivate


User = get_user_model()


class TopicForPrivateForm(forms.ModelForm):

    class Meta:
        model = Topic
        fields = ('title', )

    def __init__(self, user=None, *args, **kwargs):
        super(TopicForPrivateForm, self).__init__(*args, **kwargs)
        self.user = user

    def save(self, commit=True):
        if not self.instance.pk:
            self.instance.user = self.user
            self.instance.category = Category.objects.get(pk=settings.ST_TOPIC_PRIVATE_CATEGORY_PK)

        return super(TopicForPrivateForm, self).save(commit)


class TopicPrivateManyForm(forms.Form):

    # Only good for create
    users = forms.ModelMultipleChoiceField(queryset=User.objects.all(),
                                           to_field_name=User.USERNAME_FIELD,
                                           widget=MultipleInput(attrs={'placeholder': _("user1, user2, ..."), }))

    def __init__(self, user=None, topic=None, *args, **kwargs):
        super(TopicPrivateManyForm, self).__init__(*args, **kwargs)
        self.user = user
        self.topic = topic

    def clean_users(self):
        users = set(self.cleaned_data['users'])

        if self.user not in users:
            users.add(self.user)

        return users

    def save_m2m(self):
        users = self.cleaned_data['users']
        # Since the topic was just created this should not raise an exception
        return TopicPrivate.objects.bulk_create([TopicPrivate(user=user, topic=self.topic)
                                                 for user in users])


class TopicPrivateInviteForm(forms.ModelForm):

    # Only good for create
    user = forms.ModelChoiceField(queryset=User.objects.all(),
                                  to_field_name=User.USERNAME_FIELD,
                                  widget=forms.TextInput(attrs={'placeholder': _("username"), }),
                                  label=_("Invite user"))

    def __init__(self, topic=None, *args, **kwargs):
        super(TopicPrivateInviteForm, self).__init__(*args, **kwargs)
        self.topic = topic

    class Meta:
        model = TopicPrivate
        fields = ("user", )

    def clean_user(self):
        user = self.cleaned_data['user']

        private = TopicPrivate.objects.filter(user=user,
                                              topic=self.topic)

        if private.exists():
            # Do this since some of the unique_together fields are excluded.
            raise forms.ValidationError(_("%(username)s is already a participant") %
                                        {'username': getattr(user, user.USERNAME_FIELD), })

        return user

    def save(self, commit=True):
        if not self.instance.pk:
            self.instance.topic = self.topic

        return super(TopicPrivateInviteForm, self).save(commit)


class TopicPrivateJoinForm(forms.ModelForm):

    def __init__(self, topic=None, user=None, *args, **kwargs):
        super(TopicPrivateJoinForm, self).__init__(*args, **kwargs)
        self.topic = topic
        self.user = user

    class Meta:
        model = TopicPrivate
        fields = ()

    def clean(self):
        cleaned_data = super(TopicPrivateJoinForm, self).clean()

        private = TopicPrivate.objects.filter(user=self.user,
                                              topic=self.topic)

        if private.exists():
            # Do this since some of the unique_together fields are excluded.
            raise forms.ValidationError(_("%(username)s is already a participant") %
                                        {'username': getattr(self.user, self.user.USERNAME_FIELD), })

        return cleaned_data

    def save(self, commit=True):
        if not self.instance.pk:
            self.instance.topic = self.topic
            self.instance.user = self.user

        return super(TopicPrivateJoinForm, self).save(commit)


#-*- coding: utf-8 -*-

from django.conf.urls import patterns, url


urlpatterns = patterns("spirit.views.comment",
    url(r'^(?P<topic_id>\d+)/publish/$', 'comment_publish', name='comment-publish'),
    url(r'^(?P<topic_id>\d+)/publish/(?P<pk>\d+)/quote/$', 'comment_publish', name='comment-publish'),

    url(r'^(?P<pk>\d+)/update/$', 'comment_update', name='comment-update'),
    url(r'^(?P<pk>\d+)/find/$', 'comment_find', name='comment-find'),
    url(r'^(?P<topic_id>\d+)/move/$', 'comment_move', name='comment-move'),

    url(r'^(?P<pk>\d+)/delete/$', 'comment_delete', name='comment-delete'),
    url(r'^(?P<pk>\d+)/undelete/$', 'comment_delete', kwargs={'remove': False, }, name='comment-undelete'),

    url(r'^upload/$', 'comment_image_upload_ajax', name='comment-image-upload-ajax'),
)


#-*- coding: utf-8 -*-

from django.db import models
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse

from ..managers.comment_like import CommentLikeManager


class CommentLike(models.Model):

    user = models.ForeignKey(settings.AUTH_USER_MODEL)
    comment = models.ForeignKey('spirit.Comment', related_name='comment_likes')

    date = models.DateTimeField(auto_now_add=True)

    objects = CommentLikeManager()

    class Meta:
        app_label = 'spirit'
        unique_together = ('user', 'comment')
        ordering = ['-date', ]
        verbose_name = _("like")
        verbose_name_plural = _("likes")

    def get_delete_url(self):
        return reverse('spirit:like-delete', kwargs={'pk': str(self.pk), })

    def __unicode__(self):
        return "%s likes %s" % (self.user, self.comment)


#-*- coding: utf-8 -*-

from haystack.views import SearchView as BaseSearchView


class SearchView(BaseSearchView):

    def build_page(self):
        paginator = None
        page = self.results
        return paginator, page


#-*- coding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse
from django.conf import settings
from django.db.models import F

from ..signals.comment_like import comment_like_post_create, comment_like_post_delete
from ..signals.topic import topic_post_moderate

from spirit.managers.comment import CommentManager
from ..signals.comment import comment_post_update


COMMENT_MAX_LEN = 3000  # changing this needs migration

COMMENT, MOVED, CLOSED, UNCLOSED, PINNED, UNPINNED = xrange(6)

ACTION = (
    (COMMENT, _("comment")),
    (MOVED, _("topic moved")),
    (CLOSED, _("topic closed")),
    (UNCLOSED, _("topic unclosed")),
    (PINNED, _("topic pinned")),
    (UNPINNED, _("topic unpinned")),
)


class Comment(models.Model):

    user = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name=_("user"))
    topic = models.ForeignKey('spirit.Topic')

    comment = models.TextField(_("comment"), max_length=COMMENT_MAX_LEN)
    comment_html = models.TextField(_("comment html"))
    action = models.IntegerField(_("action"), choices=ACTION, default=COMMENT)
    date = models.DateTimeField(auto_now_add=True)
    is_removed = models.BooleanField(default=False)
    is_modified = models.BooleanField(default=False)
    ip_address = models.GenericIPAddressField(blank=True, null=True)

    modified_count = models.PositiveIntegerField(_("modified count"), default=0)
    likes_count = models.PositiveIntegerField(_("likes count"), default=0)

    objects = CommentManager()

    class Meta:
        app_label = 'spirit'
        ordering = ['-date', ]
        verbose_name = _("comment")
        verbose_name_plural = _("comments")

    def get_absolute_url(self):
        return reverse('spirit:comment-find', kwargs={'pk': str(self.id), })

    def __unicode__(self):
        return "%s: %s..." % (self.user.username, self.comment[:50])


def comment_post_update_handler(sender, comment, **kwargs):
    Comment.objects.filter(pk=comment.pk).update(modified_count=F('modified_count') + 1)


def comment_like_post_create_handler(sender, comment, **kwargs):
    Comment.objects.filter(pk=comment.pk).update(likes_count=F('likes_count') + 1)


def comment_like_post_delete_handler(sender, comment, **kwargs):
    Comment.objects.filter(pk=comment.pk).update(likes_count=F('likes_count') - 1)


def topic_post_moderate_handler(sender, user, topic, action, **kwargs):
    Comment.objects.create(user=user, topic=topic, action=action, comment="action", comment_html="action")


comment_post_update.connect(comment_post_update_handler, dispatch_uid=__name__)
comment_like_post_create.connect(comment_like_post_create_handler, dispatch_uid=__name__)
comment_like_post_delete.connect(comment_like_post_delete_handler, dispatch_uid=__name__)
topic_post_moderate.connect(topic_post_moderate_handler, dispatch_uid=__name__)


#-*- coding: utf-8 -*-

from django.core.cache import cache
from django.test import TestCase, RequestFactory
from django.template import Template, Context, TemplateSyntaxError
from django.contrib.auth.models import AnonymousUser, User
from django.contrib.messages.storage.fallback import FallbackStorage

from spirit.utils.ratelimit import RateLimit
from spirit.utils.ratelimit.decorators import ratelimit


def setup_request_factory_messages(req):
    # RequestFactory: It does not support middleware.
    # Session and authentication attributes must be supplied
    # by the test itself if required for the view to function properly.
    setattr(req, 'session', 'session')
    messages = FallbackStorage(req)
    setattr(req, '_messages', messages)


class UtilsRateLimitTests(TestCase):

    def setUp(self):
        cache.clear()

    def test_rate_limit_split_rate(self):
        req = RequestFactory().post('/')
        req.user = AnonymousUser()
        rl = RateLimit(req, 'func_name')
        self.assertEqual(rl.split_rate('5/m'), (5, 60))
        self.assertEqual(rl.split_rate('5/5m'), (5, 60 * 5))
        self.assertEqual(rl.split_rate('5/s'), (5, 1))
        self.assertEqual(rl.split_rate('5/5s'), (5, 1 * 5))
        self.assertEqual(rl.split_rate('5/15s'), (5, 15))
        self.assertEqual(rl.split_rate('15/15s'), (15, 15))

    def test_rate_limit_user_or_ip(self):
        req = RequestFactory().get('/')
        req.user = AnonymousUser()
        setup_request_factory_messages(req)

        @ratelimit(method=['GET', ], rate='1/m')
        def limit_ip(request):
            return request.is_limited

        self.assertFalse(limit_ip(req))
        self.assertTrue(limit_ip(req))

        req.user = User()
        req.user.pk = 1

        self.assertFalse(limit_ip(req))
        self.assertTrue(limit_ip(req))

    def test_rate_limit_method(self):
        rf = RequestFactory()
        post = rf.post('/')
        get = rf.get('/')
        post.user = AnonymousUser()
        get.user = AnonymousUser()
        setup_request_factory_messages(get)
        setup_request_factory_messages(post)

        @ratelimit(method=['POST', ], rate='1/m')
        def limit_post(request):
            return request.is_limited

        self.assertFalse(limit_post(post))
        self.assertTrue(limit_post(post))
        self.assertFalse(limit_post(get))

    def test_rate_limit_field(self):
        req = RequestFactory().post('/', {'username': 'esteban', })
        req.user = AnonymousUser()
        setup_request_factory_messages(req)

        @ratelimit(field='username', rate='1/m')
        def username(request):
            return request.is_limited

        self.assertFalse(username(req))
        self.assertTrue(username(req))

        req.user = User()
        req.user.pk = 1

        self.assertTrue(username(req))

    def test_rate_limit_field_empty(self):
        empty = RequestFactory().post('/', {'username': '', })
        empty.user = AnonymousUser()

        @ratelimit(field='username', rate='1/m')
        def username(request):
            return request.is_limited

        self.assertFalse(username(empty))

        empty.user = User()
        empty.user.pk = 1

        self.assertFalse(username(empty))

    def test_rate_limit_rate(self):
        req = RequestFactory().post('/')
        req.user = AnonymousUser()
        setup_request_factory_messages(req)

        @ratelimit(rate='2/m')
        def two(request):
            return request.is_limited

        self.assertFalse(two(req))
        self.assertFalse(two(req))
        self.assertTrue(two(req))


#-*- coding: utf-8 -*-

from django.contrib.auth.decorators import login_required
from django.shortcuts import render, redirect, get_object_or_404
from django.views.decorators.http import require_POST
from django.contrib import messages
from django.utils.translation import ugettext as _
from django.conf import settings
from django.http import Http404

from spirit.utils.ratelimit.decorators import ratelimit
from spirit.models.topic import Topic
from spirit.utils import paginator, markdown, render_form_errors
from spirit.utils.decorators import moderator_required
from spirit.utils import json_response, render_form_errors

from spirit.models.comment import Comment
from spirit.forms.comment import CommentForm, CommentMoveForm, CommentImageForm
from spirit.signals.comment import comment_posted, comment_pre_update, comment_post_update, comment_moved


@login_required
@ratelimit(rate='1/10s')
def comment_publish(request, topic_id, pk=None):
    topic = get_object_or_404(Topic.objects.for_access_open(request.user),
                              pk=topic_id)

    if request.method == 'POST':
        form = CommentForm(user=request.user, topic=topic, data=request.POST)

        if not request.is_limited and form.is_valid():
            comment = form.save()
            comment_posted.send(sender=comment.__class__, comment=comment, mentions=form.mentions)
            return redirect(request.POST.get('next', comment.get_absolute_url()))
    else:
        initial = None

        if pk:
            comment = get_object_or_404(Comment.objects.for_all(), pk=pk)
            quote = markdown.quotify(comment.comment, comment.user.username)
            initial = {'comment': quote, }

        form = CommentForm(initial=initial)

    return render(request, 'spirit/comment/comment_publish.html', {'form': form, 'topic': topic})


@login_required
def comment_update(request, pk):
    comment = Comment.objects.for_update_or_404(pk, request.user)

    if request.method == 'POST':
        form = CommentForm(data=request.POST, instance=comment)

        if form.is_valid():
            comment_pre_update.send(sender=comment.__class__, comment=Comment.objects.get(pk=comment.pk))
            comment = form.save()
            comment_post_update.send(sender=comment.__class__, comment=comment)
            return redirect(request.POST.get('next', comment.get_absolute_url()))
    else:
        form = CommentForm(instance=comment)

    return render(request, 'spirit/comment/comment_update.html', {'form': form, })


@moderator_required
def comment_delete(request, pk, remove=True):
    comment = get_object_or_404(Comment, pk=pk)

    if request.method == 'POST':
        Comment.objects.filter(pk=pk)\
            .update(is_removed=remove)

        return redirect(comment.get_absolute_url())

    return render(request, 'spirit/comment/comment_moderate.html', {'comment': comment, })


@require_POST
@moderator_required
def comment_move(request, topic_id):
    topic = get_object_or_404(Topic, pk=topic_id)
    form = CommentMoveForm(topic=topic, data=request.POST)

    if form.is_valid():
        comments = form.save()

        for comment in comments:
            comment_posted.send(sender=comment.__class__, comment=comment, mentions=None)

        comment_moved.send(sender=Comment, comments=comments, topic_from=topic)
    else:
        messages.error(request, render_form_errors(form))

    return redirect(request.POST.get('next', topic.get_absolute_url()))


def comment_find(request, pk):
    comment = get_object_or_404(Comment, pk=pk)
    comment_number = Comment.objects.filter(topic=comment.topic, date__lte=comment.date).count()
    url = paginator.get_url(comment.topic.get_absolute_url(),
                            comment_number,
                            settings.ST_COMMENTS_PER_PAGE,
                            settings.ST_COMMENTS_PAGE_VAR)
    return redirect(url)


@require_POST
@login_required
def comment_image_upload_ajax(request):
    if not request.is_ajax():
        return Http404()

    form = CommentImageForm(user=request.user, data=request.POST, files=request.FILES)

    if form.is_valid():
        image = form.save()
        return json_response({'url': image.url, })

    return json_response({'error': dict(form.errors.items()), })


from category import Category
from comment import Comment
from comment_history import CommentHistory
from comment_bookmark import CommentBookmark
from comment_flag import Flag
from comment_like import CommentLike
from topic import Topic
from topic_unread import TopicUnread
from topic_favorite import TopicFavorite
from topic_notification import TopicNotification
from topic_poll import TopicPoll, TopicPollChoice
from topic_private import TopicPrivate
from user import User


#-*- coding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse
from django.conf import settings
from django.db.models import F
from django.utils import timezone

from ..signals.comment import comment_posted, comment_moved

from spirit.signals.topic import topic_viewed
from spirit.managers.topic import TopicManager
from spirit.utils.models import AutoSlugField


class Topic(models.Model):

    user = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name=_("user"))
    category = models.ForeignKey('spirit.Category', verbose_name=_("category"))

    title = models.CharField(_("title"), max_length=75)
    slug = AutoSlugField(populate_from="title", db_index=False, blank=True)
    date = models.DateTimeField(_("date"), auto_now_add=True)
    last_active = models.DateTimeField(_("last active"), auto_now_add=True)
    is_pinned = models.BooleanField(_("pinned"), default=False)
    is_closed = models.BooleanField(_("closed"), default=False)
    is_removed = models.BooleanField(default=False)

    view_count = models.PositiveIntegerField(_("views count"), default=0)
    comment_count = models.PositiveIntegerField(_("comment count"), default=0)

    objects = TopicManager()

    class Meta:
        app_label = 'spirit'
        ordering = ['-last_active', ]
        verbose_name = _("topic")
        verbose_name_plural = _("topics")

    def get_absolute_url(self):
        if self.category_id == settings.ST_TOPIC_PRIVATE_CATEGORY_PK:
            return reverse('spirit:private-detail', kwargs={'topic_id': str(self.id), 'slug': self.slug})

        return reverse('spirit:topic-detail', kwargs={'pk': str(self.id), 'slug': self.slug})

    @property
    def main_category(self):
        return self.category.parent or self.category

    def __unicode__(self):
        return self.title


def topic_page_viewed_handler(sender, request, topic, **kwargs):
    Topic.objects.filter(pk=topic.pk)\
        .update(view_count=F('view_count') + 1)


def comment_posted_handler(sender, comment, **kwargs):
    Topic.objects.filter(pk=comment.topic.pk)\
        .update(comment_count=F('comment_count') + 1, last_active=timezone.now())


def comment_moved_handler(sender, comments, topic_from, **kwargs):
    Topic.objects.filter(pk=topic_from.pk)\
        .update(comment_count=F('comment_count') - len(comments))


topic_viewed.connect(topic_page_viewed_handler, dispatch_uid=__name__)
comment_posted.connect(comment_posted_handler, dispatch_uid=__name__)
comment_moved.connect(comment_moved_handler, dispatch_uid=__name__)


#-*- coding: utf-8 -*-

from django.conf.urls import patterns, include, url


urlpatterns = patterns('',
    url(r'^$', 'spirit.views.admin.index.dashboard', name='admin'),
    url(r'^index/', include('spirit.urls.admin.index')),
    url(r'^category/', include('spirit.urls.admin.category')),
    url(r'^comment/flag/', include('spirit.urls.admin.comment_flag')),
    url(r'^config/', include('spirit.urls.admin.config')),
    url(r'^topic/', include('spirit.urls.admin.topic')),
    url(r'^user/', include('spirit.urls.admin.user')),
)


#-*- coding: utf-8 -*-

# THIS IS FOR DEVELOPMENT ENVIRONMENT, MOSTLY TO SPEED UP TESTS
# DO NOT USE IT IN PRODUCTION

import os
import sys


DEBUG = True

TEMPLATE_DEBUG = True

ALLOWED_HOSTS = ['127.0.0.1', ]

BASE_DIR = os.path.dirname(os.path.dirname(__file__))

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
    },
    'djconfig': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
    },
}

PASSWORD_HASHERS = (
    'django.contrib.auth.hashers.MD5PasswordHasher',
)

EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'

TESTING = 'test' in sys.argv

if TESTING:
    # Keep templates in memory
    TEMPLATE_LOADERS = (
        ('django.template.loaders.cached.Loader', (
            'django.template.loaders.filesystem.Loader',
            'django.template.loaders.app_directories.Loader',
        )),
    )
else:
    TEMPLATE_LOADERS = (
        'django.template.loaders.filesystem.Loader',
        'django.template.loaders.app_directories.Loader',
    )


#-*- coding: utf-8 -*-

from django import forms
from django.utils.translation import ugettext_lazy as _

from spirit.models.topic_favorite import TopicFavorite


class FavoriteForm(forms.ModelForm):

    class Meta:
        model = TopicFavorite
        fields = []

    def __init__(self, user=None, topic=None, *args, **kwargs):
        super(FavoriteForm, self).__init__(*args, **kwargs)
        self.user = user
        self.topic = topic

    def clean(self):
        cleaned_data = super(FavoriteForm, self).clean()

        favorite = TopicFavorite.objects.filter(user=self.user,
                                                topic=self.topic)

        if favorite.exists():
            # Do this since some of the unique_together fields are excluded.
            raise forms.ValidationError(_("This favorite already exists"))

        return cleaned_data

    def save(self, commit=True):
        if not self.instance.pk:
            self.instance.user = self.user
            self.instance.topic = self.topic

        return super(FavoriteForm, self).save(commit)


#-*- coding: utf-8 -*-

from django.test import TestCase
from django.db import models

from spirit.utils.models import AutoSlugField


class AutoSlugModel(models.Model):

    slug = AutoSlugField(max_length=50)

    class Meta:
        app_label = 'spirit'


class AutoSlugDefaultModel(models.Model):

    slug = AutoSlugField(max_length=50, default="foo")

    class Meta:
        app_label = 'spirit'


class AutoSlugBadPopulateFromModel(models.Model):

    slug = AutoSlugField(populate_from='bad', max_length=50)

    class Meta:
        app_label = 'spirit'


class AutoSlugPopulateFromModel(models.Model):

    title = models.CharField(max_length=255, blank=True)
    slug = AutoSlugField(populate_from='title', max_length=50)

    class Meta:
        app_label = 'spirit'


class UtilsModelsTests(TestCase):

    def test_auto_slug_field(self):
        """
        Should behave like a regular SlugField if populate_from is not provided
        """
        foo_model = AutoSlugModel(slug="foo")
        foo_model.save()
        self.assertEqual(foo_model.slug, "foo")

        foo_model = AutoSlugModel()
        foo_model.save()
        self.assertEqual(foo_model.slug, "")

    def test_auto_slug_field_default(self):
        """
        AutoSlugField default
        """
        foo_model = AutoSlugDefaultModel()
        foo_model.save()
        self.assertEqual(foo_model.slug, "foo")

        foo_model = AutoSlugDefaultModel(slug="bar")
        foo_model.save()
        self.assertEqual(foo_model.slug, "bar")

    def test_auto_slug_field_invalid_populate_from(self):
        """
        Should raise AttributeError
        """
        foo_model = AutoSlugBadPopulateFromModel()
        self.assertRaisesMessage(AttributeError, "'AutoSlugBadPopulateFromModel' "
                                                 "object has no attribute 'bad'", foo_model.save)

    def test_auto_slug_field_populate_from(self):
        """
        AutoSlugField populate_from
        """
        title = "a" * 255
        foo_model = AutoSlugPopulateFromModel(title=title)
        foo_model.save()
        self.assertEqual(foo_model.slug, title[:50])

        # update title
        foo_model.title = "foo"
        foo_model.save()
        self.assertEqual(foo_model.slug, title[:50])

        # update slug
        foo_model.slug = "foo"
        foo_model.save()
        self.assertEqual(foo_model.slug, "foo")

        # update slug to blank
        foo_model.slug = ""
        foo_model.save()
        self.assertEqual(foo_model.slug, "")

        # populate_from field is blank
        foo_model = AutoSlugPopulateFromModel()
        self.assertEqual(foo_model.slug, "")


#-*- coding: utf-8 -*-

from django.shortcuts import render, redirect, get_object_or_404
from django.contrib.auth import get_user_model
from django.contrib import messages
from django.utils.translation import ugettext as _

from spirit.utils.decorators import administrator_required

from spirit.forms.admin import UserEditForm


User = get_user_model()


@administrator_required
def user_edit(request, user_id):
    user = get_object_or_404(User, pk=user_id)

    if request.method == 'POST':
        form = UserEditForm(data=request.POST, instance=user)

        if form.is_valid():
            form.save()
            messages.info(request, _("This profile has been updated!"))
            return redirect(request.GET.get("next", request.get_full_path()))
    else:
        form = UserEditForm(instance=user)

    return render(request, 'spirit/admin/user/user_edit.html', {'form': form, })


@administrator_required
def user_list(request):
    users = User.objects.all()
    return render(request, 'spirit/admin/user/user_list.html', {'users': users, })


@administrator_required
def user_admins(request):
    users = User.objects.filter(is_administrator=True)
    return render(request, 'spirit/admin/user/user_admins.html', {'users': users, })


@administrator_required
def user_mods(request):
    users = User.objects.filter(is_moderator=True, is_administrator=False)
    return render(request, 'spirit/admin/user/user_mods.html', {'users': users, })


@administrator_required
def user_unactive(request):
    users = User.objects.filter(is_active=False)
    return render(request, 'spirit/admin/user/user_unactive.html', {'users': users, })


#-*- coding: utf-8 -*-

from django.dispatch import Signal


comment_posted = Signal(providing_args=['comment', 'mentions'])
comment_pre_update = Signal(providing_args=['comment', ])
comment_post_update = Signal(providing_args=['comment', ])
comment_moved = Signal(providing_args=['comments', 'topic_from'])


#-*- coding: utf-8 -*-

from django.shortcuts import render
from django.contrib.auth import get_user_model

from spirit.utils.decorators import administrator_required

import spirit
from spirit.models.category import Category
from spirit.models.topic import Topic
from spirit.models.comment import Comment
from spirit.models.comment_flag import CommentFlag
from spirit.models.comment_like import CommentLike


User = get_user_model()


@administrator_required
def dashboard(request):
    # Strongly unaccurate counters below...
    category_count = Category.objects.all().count() - 1  # - private
    topics_count = Topic.objects.all().count()
    comments_count = Comment.objects.all().count()
    users_count = User.objects.all().count()
    flags_count = CommentFlag.objects.filter(is_closed=False).count()
    likes_count = CommentLike.objects.all().count()

    return render(request, 'spirit/admin/index/dashboard.html', {'version': spirit.__version__,
                                                                 'category_count': category_count,
                                                                 'topics_count': topics_count,
                                                                 'comments_count': comments_count,
                                                                 'users_count': users_count,
                                                                 'flags_count': flags_count,
                                                                 'likes_count': likes_count})


#-*- coding: utf-8 -*-

from django.core import signing
from django.utils.encoding import smart_text


class TokenGenerator(object):

    def _uid(self, user):
        raise NotImplementedError

    def generate(self, user, data=None):
        """
        Django signer uses colon (:) for components separation
        JSON_object:hash_first_component:hash_secret, all base64 encoded
        that aint so url-safe, so Im replacing them by dots (.)

        base64 encode characters ref: 0-9, A-Z, a-z, _, -
        """
        data = data or {}
        data.update({'uid': self._uid(user), })
        return signing.dumps(data, salt=__name__).replace(":", ".")

    def is_valid(self, user, signed_value):
        try:
            self.data = signing.loads(signed_value.replace(".", ":"), salt=__name__)
        except signing.BadSignature:
            return False

        if self.data['uid'] != self._uid(user):
            return False

        return True


class UserActivationTokenGenerator(TokenGenerator):

    def _uid(self, user):
        # Older mysql won't store ms.
        return u";".join((smart_text(user.pk), smart_text(user.last_login.replace(microsecond=0))))


class UserEmailChangeTokenGenerator(TokenGenerator):

    def _uid(self, user):
        return u";".join((smart_text(user.pk), smart_text(user.email)))

    def generate(self, user, new_email):
        return super(UserEmailChangeTokenGenerator, self).generate(user, {'new_email': new_email, })

    def get_email(self):
        return self.data['new_email']


#-*- coding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse
from django.conf import settings
from django.utils import timezone
from django.db import IntegrityError

from spirit.signals.comment import comment_posted

from spirit.managers.topic_unread import TopicUnreadManager
from spirit.signals.topic import topic_viewed


class TopicUnread(models.Model):

    user = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name=_("user"))
    topic = models.ForeignKey('spirit.Topic')

    date = models.DateTimeField(auto_now_add=True)
    is_read = models.BooleanField(default=True)

    objects = TopicUnreadManager()

    class Meta:
        app_label = 'spirit'
        unique_together = ('user', 'topic')
        ordering = ['-date', ]
        verbose_name = _("topic unread")
        verbose_name_plural = _("topics unread")

    def get_absolute_url(self):
        return self.topic.get_absolute_url()

    def __unicode__(self):
        return "%s read %s" % (self.user, self.topic)


def topic_page_viewed_handler(sender, request, topic, **kwargs):
    if not request.user.is_authenticated():
        return

    # TODO: use update_or_create on django 1.7
    try:
        TopicUnread.objects.create(user=request.user, topic=topic)
    except IntegrityError:
        TopicUnread.objects.filter(user=request.user, topic=topic)\
            .update(is_read=True)


def comment_posted_handler(sender, comment, **kwargs):
    TopicUnread.objects.filter(topic=comment.topic)\
        .exclude(user=comment.user)\
        .update(is_read=False, date=timezone.now())


topic_viewed.connect(topic_page_viewed_handler, dispatch_uid=__name__)
comment_posted.connect(comment_posted_handler, dispatch_uid=__name__)


#-*- coding: utf-8 -*-

from django.conf.urls import patterns, url


urlpatterns = patterns("spirit.views.topic",
    url(r'^publish/$', 'topic_publish', name='topic-publish'),
    url(r'^publish/(?P<category_id>\d+)/$', 'topic_publish', name='topic-publish'),

    url(r'^update/(?P<pk>\d+)/$', 'topic_update', name='topic-update'),

    url(r'^(?P<pk>\d+)/$', 'topic_detail', kwargs={'slug': "", }, name='topic-detail'),
    url(r'^(?P<pk>\d+)/(?P<slug>[\w-]+)/$', 'topic_detail', name='topic-detail'),

    url(r'^active/$', 'topics_active', name='topic-active'),

    url(r'^delete/(?P<pk>\d+)/$', 'topic_moderate', kwargs={'remove': True, 'value': True}, name='topic-delete'),
    url(r'^undelete/(?P<pk>\d+)/$', 'topic_moderate', kwargs={'remove': True, 'value': False}, name='topic-undelete'),

    url(r'^lock/(?P<pk>\d+)/$', 'topic_moderate', kwargs={'lock': True, 'value': True}, name='topic-lock'),
    url(r'^unlock/(?P<pk>\d+)/$', 'topic_moderate', kwargs={'lock': True, 'value': False}, name='topic-unlock'),

    url(r'^pin/(?P<pk>\d+)/$', 'topic_moderate', kwargs={'pin': True, 'value': True}, name='topic-pin'),
    url(r'^unpin/(?P<pk>\d+)/$', 'topic_moderate', kwargs={'pin': True, 'value': False}, name='topic-unpin'),
)


#-*- coding: utf-8 -*-

import urllib
import hashlib

from .. import register


@register.simple_tag()
def get_gravatar_url(user, size, rating='g', default='identicon'):
    url = "http://www.gravatar.com/avatar/"
    hash = hashlib.md5(user.email.strip().lower()).hexdigest()
    data = urllib.urlencode({'d': urllib.quote(default), 's': str(size), 'r': rating})
    return "".join((url, hash, '?', data))



#-*- coding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse
from django.conf import settings
from django.db.models import signals

from spirit.signals.comment import comment_pre_update, comment_post_update


class CommentHistory(models.Model):

    comment_fk = models.ForeignKey('spirit.Comment', verbose_name=_("original comment"))

    comment_html = models.TextField(_("comment html"))
    date = models.DateTimeField(auto_now_add=True)

    class Meta:
        app_label = 'spirit'
        ordering = ['-date', ]
        verbose_name = _("comment history")
        verbose_name_plural = _("comments history")

    def get_absolute_url(self):
        return reverse('spirit:comment-history', kwargs={'pk': str(self.id), })

    def __unicode__(self):
        return "%s: %s..." % (self.comment_fk.user.username, self.comment_html[:50])


def comment_pre_update_handler(sender, comment, **kwargs):
    # save original comment
    exists = CommentHistory.objects.filter(comment_fk=comment).exists()

    if not exists:
        CommentHistory.objects.create(comment_fk=comment, comment_html=comment.comment_html, date=comment.date)


def comment_post_update_handler(sender, comment, **kwargs):
    CommentHistory.objects.create(comment_fk=comment, comment_html=comment.comment_html)


comment_pre_update.connect(comment_pre_update_handler, dispatch_uid=__name__)
comment_post_update.connect(comment_post_update_handler, dispatch_uid=__name__)


#-*- coding: utf-8 -*-

import os
import json

from django.template.loader import render_to_string
from django.http import HttpResponse


def render_form_errors(form):
    return render_to_string('spirit/utils/_form_errors.html', {'form': form, })


def json_response(data=None, status=200):
    # TODO: Use JsonResponse on Django 1.7
    data = data or {}
    return HttpResponse(json.dumps(data), content_type='application/json', status=status)


def mkdir_p(path):
    try:
        os.makedirs(path)
    except OSError:
        if not os.path.isdir(path):
            raise


#-*- coding: utf-8 -*-

import re

from markdown.extensions import Extension
from markdown.preprocessors import Preprocessor


class ImagifyExtension(Extension):

    def extendMarkdown(self, md, md_globals):
        md.registerExtension(self)
        md.preprocessors.add('imagify',
                             ImagifyPreprocessor(md),
                             '_end')


class ImagifyPreprocessor(Preprocessor):

    def run(self, lines):
        new_lines = []

        def imagify(match):
            return u'![image](%s)' % match.group(0)

        for line in lines:
            if line.strip():
                line = re.sub(ur'^https?://[^\s]+/(?P<image_name>[^\s]+)\.'
                              ur'(?P<extension>png|jpg|jpeg|gif|bmp|tif|tiff)'
                              ur'(\?[^\s]+)?$', imagify, line, flags=re.UNICODE)

            new_lines.append(line)

        return new_lines


def makeExtension(configs=None):
    return ImagifyExtension(configs=configs)